#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <dirent.h>
#include <rsbac/types.h>
#include <rsbac/getname.h>
#include <rsbac/syscalls.h>
#include <rsbac/error.h>
#include <rsbac/helpers.h>
#include <rsbac/aci_data_structures.h>
#include "locale.h"

const char   set_prog[] = "attr_set_file_dir";
enum rsbac_attribute_t attr_list[RSBAC_DEV_NR_ATTRIBUTES] = RSBAC_DEV_ATTR_LIST;
int recurse = 0;
int verbose = 0;
FILE * tfile;
char * filename = NULL;

int process(char * name)
  {
    struct rsbac_dev_aci_t default_dev_aci = DEFAULT_DEV_ACI;
    __s64 def_attr[RSBAC_DEV_NR_ATTRIBUTES] = {
      default_dev_aci.sec_level,
      default_dev_aci.mac_categories,
      default_dev_aci.object_category,
      default_dev_aci.data_type,
      default_dev_aci.mac_check,
      default_dev_aci.pm_object_type,
      default_dev_aci.pm_object_class,
      default_dev_aci.rc_type
      };
    int res = 0;
    char tmp1[120];
    char tmp2[65];
    int j;
    struct stat buf;
    union rsbac_attribute_value_t value;

    if (lstat(name,&buf))
      {
        fprintf(stderr, gettext("stat for file %s returned error: %s\n"),
               name,
               strerror(errno));
        return(1);
      }
      if(   !(S_ISCHR(buf.st_mode))
         && !(S_ISBLK(buf.st_mode)) )
        {
          if(verbose)
            printf(gettext("%s is neither block nor char special file, skipped\n"),
                   name);
          return(-1);
        }
    if(verbose)
      printf(gettext("Processing DEV '%s'\n"), name);
    for (j=0;j < RSBAC_DEV_NR_ATTRIBUTES;j++)
      {
        value.dummy = -1;
        res = rsbac_get_attr_n(T_DEV, name, attr_list[j], &value, 0);
        if(res)
          {
            get_error_name(tmp1,res);
            fprintf(stderr, "%s (%s): %s\n",
                    name,
                    get_attribute_name(tmp2,attr_list[j]),
                    tmp1);
          }
        else
          switch(attr_list[j])
            {
              case A_log_array_low:
              case A_log_array_high:
                if (value.log_array_low != -1)
                  fprintf(tfile,
                          "%s DEV %s %s %s\n",
                          set_prog,
                          name,
                          get_attribute_name(tmp1,attr_list[j]),
                          u64tostrlog(tmp2,value.log_array_low));
                break;
              case A_security_level:
              case A_rc_type:
                if(value.security_level != def_attr[j])
                  fprintf(tfile,
                          "%s DEV %s %s %u\n",
                          set_prog,
                          name,
                          get_attribute_name(tmp1,attr_list[j]),
                          value.security_level);
                break;
              default:
                if(value.dummy != def_attr[j])
                  fprintf(tfile,
                          "%s DEV %s %s %i\n",
                          set_prog,
                          name,
                          get_attribute_name(tmp1,attr_list[j]),
                          value.dummy);
            }
      }
    return(res);
  }

int main(int argc, char ** argv)
{
  int res = 0;
  char tmp1[80],tmp2[120];
  int i,j;
  char * progname;

  setlocale (LC_ALL, "");
  bindtextdomain (PACKAGE, LOCALEDIR);
  textdomain (PACKAGE);

  progname = argv[0];
  while((argc > 1) && (argv[1][0] == '-'))
    {
      char * pos = argv[1];
      pos++;
      while(*pos)
        {
          switch(*pos)
            {
              case 'v':
                verbose++;
                break;
              case 'o':
                if(argc > 2)
                  {
                    filename = argv[2];
                    argv++;
                    argc--;
                  }
                else
                  fprintf(stderr, gettext("%s: missing filename for parameter o\n"), progname);
                break;
              default:
                fprintf(stderr, gettext("%s: unknown parameter %c\n"), progname, *pos);
            }
          pos++;
        }
      argv++;
      argc--;
    }

  if (argc > 1)
    {
      if(!filename)
        tfile = stdout;
      else
        {
          if (!(tfile=fopen(filename,"w")))
            {
              fprintf(stderr, gettext("opening target file returned error: %s\n"),
                              strerror(errno));
            }
        }
      if(verbose)
        printf(gettext("%s: %i targets\n"), progname, argc - 1);
      for (i=1;i < argc;i++)
        {
          process(argv[i]);
        }
      if(tfile != stdout)
        fclose(tfile);
    }
  else
    {
      printf(gettext("%s (RSBAC %s)\n***\n"), argv[0], VERSION);
      printf(gettext("Use: %s [-v] [-o target-file] file/dirname(s)\n"), progname);  
      printf(gettext("- should be called by root with all rsbac modules switched off,\n"));
      printf(gettext("- -v = verbose, no symlinks followed,\n"));
      printf(gettext("- -o target-file = write to file, not stdout,\n"));
      printf(gettext("- attributes and values in backup = see following list:\n"));
      for (j=0;j<RSBAC_DEV_NR_ATTRIBUTES;j++)
        {
          get_attribute_name(tmp1,attr_list[j]);
          get_attribute_param(tmp2,attr_list[j]);
          printf("%s \t%s\n",tmp1,tmp2);
        }
    }
  return (res);
}

