/*RSBAC global network
 *
 *(c) Stanislav Ievlev <inger@linux.ru.net>
 */
 
/*r-job-lib.c - r-job library*/

#include <sys/types.h>
#include <sys/socket.h>
#include <stdio.h>
#include <stdlib.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>
#include <unistd.h>
#include <string.h>

#include "r-job.h"
#include "auth.h"
#include "rsa.h"


char command_list[NR_COMMAND][MAX_LINE]={GET_ONLY_COMMAND,
				GET_ONLY_WARRANTY_COMMAND,
				ADD_ONLY_COMMAND,
				ADD_ONLY_MINE_COMMAND,
				GET_AND_ADD_COMMAND,
				GET_AND_ADD_WARRANTY_COMMAND,				
				CLEAN_COMMAND,
				CLEAN_ONE_STATION_COMMAND,
				PEEK_ONE_STATION_COMMAND,
				GET_ONLY_ONE_STATION_COMMAND,
				STOP_SERVER_COMMAND,
				PAUSE_SERVER_COMMAND,
				RESUME_SERVER_COMMAND,
				LIST_STATION_COMMAND,
				ADD_STATION_COMMAND,
				DEL_STATION_COMMAND};


/*sending command to server*/
void client_write_command(int s,char *message)
{
    if (send(s, message, MAX_LINE,0) ==-1){
	perror("send");
	//exit(1);
    }
#ifdef TEST
//    printf("Sending command ...%s\n",message);    
#endif
}

/*sending job to server*/
void client_write_message(int s,char *message,char *session_keyword)
{
#ifdef TEST
//    printf("Sending job...%s\n",message);
#endif
    /*encrypt message*/
    my_encrypt(session_keyword,message);
    
    /*send message*/
    if (send(s, message, MAX_LINE,0) ==-1){
	perror("send");
	//exit(1);
    }
    my_decrypt(session_keyword,message);
}

/*read jobs from server*/
void client_read_message(int s,char *session_keyword,int run,int warranty)
{
    char data_str[MAX_LINE];
    char answer[MAX_LINE];
    int result=0;
    
    while (recv(s, data_str, MAX_LINE,0) > 0){

	//decrypt message
	my_decrypt(session_keyword,data_str);

#ifdef TEST
//        printf("Recieving job ... %s\n",data_str);
#endif
        if (run){
	    result=system(data_str);
	}else{
	    fprintf(stderr,">>%s\n",data_str);
	}//run
	/*send answer if warranty needed*/
	if (warranty==1){
	    /*prepare answer*/
	    if (result==0){
		strncpy(answer,WARRANTY_YES,MAX_LINE);
		}else{
		strncpy(answer,WARRANTY_NO,MAX_LINE);
		}
#ifdef TEST
//	    fprintf(stderr,"Sending answer...%s\n",answer);
#endif
	    my_encrypt(session_keyword,answer);/*encrypt*/
	    if (send(s, answer, MAX_LINE,0) ==-1){
		perror("send");
		//exit(1);
	    };
	};//warranty
	
    }/*while*/
}

/*make connection with server*/
int client_connection(char *hostname)
{
    struct sockaddr_in serv_addr;
    struct hostent *hp;
    int s;/*socket ID*/

    client_check_cert();
    
/*resolve hostname*/
    if((hp=gethostbyname(hostname))==0)
    {
	perror("gethostbyname()");
	//exit(1);
    }

/*fill address structure*/
    bzero(&serv_addr, sizeof(serv_addr));
    bcopy(hp->h_addr,&serv_addr.sin_addr,hp->h_length);
    serv_addr.sin_family=hp->h_addrtype;
    serv_addr.sin_port=htons(PORTNUM);

    if((s=socket(AF_INET, SOCK_STREAM, 0))==-1){
	perror("socket()");
        //exit(1);
    }

/*connnect to server*/
    if (connect(s, (struct sockaddr_in *)&serv_addr, sizeof(serv_addr))==-1){
	perror("connect()");
	//exit(1);
    }
    
    return s;
}


/*get keyword for client*/
void get_keyword(char *keyword,int special)
{
    FILE *k_file=NULL;
    char *local_keyword;
    char secret[MAX_LINE];
    
    
    /*simple mode*/
    if (special==0)
	{
	local_keyword=getpass(DEFAULT_PROMPT);
	strncpy(keyword,local_keyword,MAX_LINE);
	return;
	}
    
    /*advanced mode*/	
    (void)get_secret(secret); /*get user dependence password*/
    
#ifdef TEST
//    printf("secret...%s\n",secret);
#endif

    k_file=fopen(KEY_FILE_PATH,"r");
    
    if (k_file==NULL) /*we must create new keyword file*/
    {
    
#ifdef TEST
//    printf("creating new key file\n");
#endif

    local_keyword=getpass(DEFAULT_PROMPT);
    strncpy(keyword,local_keyword,MAX_LINE);
    
    k_file=fopen(KEY_FILE_PATH,"w");
    
    if (k_file==NULL) /*bug*/
    {
	perror("fopen");
	//exit(1);
    }

    my_encrypt(secret,keyword);
    fwrite(keyword,MAX_LINE,1,k_file);
    my_decrypt(secret,keyword);    
    }else{ /*get keyword from file*/
    fread(keyword,MAX_LINE,1,k_file);
    my_decrypt(secret,keyword);
    }
    
    fclose(k_file);
}

void view_commands(FILE *f,char *prefix)
{
int i;
for (i=0;i<NR_COMMAND;i++) fprintf(f,"%s%s\n",prefix,command_list[i]);
}

void client_check_keyword(int s,char *keyword,char *session_keyword)
{
char client_keyword[MAX_LINE];

    /*RSA*/
    client_accept_station(s,"client");
    client_test(s,"client");

    /*Diffi-Helman first*/
    DH_check(s,session_keyword,0);
#ifdef TEST
//    fprintf(stderr,">>session_keyword:%s\n",session_keyword);
#endif
    
    strncpy(client_keyword,keyword,MAX_LINE); /*safe copy password*/
    my_encrypt(session_keyword,client_keyword);/*then encrypt*/
    if (send(s, client_keyword, MAX_LINE,0) ==-1){/*and send*/
	perror("send");
	//exit(1);
    }
    
}

/*read jobs from server put it to vector*/
int client_read_message_v(int s,char *session_keyword,char **answer)
{
    char data_str[MAX_LINE];
    int num=0;
    
    while (recv(s, data_str, MAX_LINE,0) > 0){
	//decrypt message
	my_decrypt(session_keyword,data_str);
#ifdef TEST
//        printf("Recieving ... %s\n",data_str);
#endif
    strncpy(answer[num],data_str,MAX_LINE);
    num++;
    }/*while*/
    
    return num;
}
