/*RSBAC global network client
 *
 *(c) Stanislav Ievlev <inger@linux.ru.net>
 */
 
/*r-job+.c - jobs client. Full command support*/

#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <fcntl.h>
#include <netdb.h>
#include <string.h>

#include "r-job.h"
#include "r-job-lib.h"
#include "auth.h"


char keyword[MAX_LINE]; /*password to work*/

char session_keyword[MAX_LINE]; /*session keyword*/

/*main work*/
int main(int argc, char *argv[])
{
    int i;
    int s;	/*connection socket*/
    char command_str[MAX_LINE];
    char puper_str[MAX_LINE];
    
/*check args*/
    if (argc<3){
	printf("Usage: r-job+ host command [job args]\n");
	printf("  host     - RSBAC server host\n");
	printf("  command  - command you want to run\n");
	printf("  job  - job to add\n");
	printf("Available commands:\n");
	view_commands(stdout,"   - ");
	exit(1);
    }
/*make command string*/
    strncpy(puper_str,argv[2],MAX_LINE);
/*make job string*/
    if (argc>3){
        strncpy(command_str,argv[3],MAX_LINE);
        for (i=4;i<argc;i++){
            strncat(command_str," ",MAX_LINE);
	    strncat(command_str,argv[i],MAX_LINE);
        }
    }else{
	strncpy(command_str,"_",MAX_LINE);
    }
    
/*get password from user*/
    get_keyword(keyword,0);

/*make connection*/
    s=client_connection(argv[1]);    
/*send password*/
    client_check_keyword(s,keyword,session_keyword);

/*send command to server*/
    client_write_command(s,puper_str);
/*send job to server*/
    if ((strcmp(puper_str,GET_ONLY_COMMAND))&&
	(strcmp(puper_str,GET_ONLY_WARRANTY_COMMAND))&&
    	(strcmp(puper_str,CLEAN_COMMAND))&&
	(strcmp(puper_str,LIST_STATION_COMMAND))&&
	(strcmp(puper_str,STOP_SERVER_COMMAND))&&
	(strcmp(puper_str,PAUSE_SERVER_COMMAND))&&
	(strcmp(puper_str,RESUME_SERVER_COMMAND))
	)
	client_write_message(s,command_str,session_keyword);
/*receive and execute commands from server*/
    if ((strcmp(puper_str,ADD_ONLY_COMMAND))&&
	(strcmp(puper_str,ADD_ONLY_MINE_COMMAND))&&
	(strcmp(puper_str,GET_ONLY_WARRANTY_COMMAND))&&
	(strcmp(puper_str,GET_AND_ADD_WARRANTY_COMMAND))&&
	(strcmp(puper_str,CLEAN_COMMAND))&&
	(strcmp(puper_str,CLEAN_ONE_STATION_COMMAND))&&
	(strcmp(puper_str,ADD_STATION_COMMAND))&&
	(strcmp(puper_str,DEL_STATION_COMMAND))&&
	(strcmp(puper_str,LIST_STATION_COMMAND))&&
	(strcmp(puper_str,PEEK_ONE_STATION_COMMAND))&&
	(strcmp(puper_str,STOP_SERVER_COMMAND))&&
	(strcmp(puper_str,PAUSE_SERVER_COMMAND))&&
	(strcmp(puper_str,RESUME_SERVER_COMMAND))
	)
        client_read_message(s,session_keyword,1,0);

    /*special case*/
    if (!(strcmp(puper_str,LIST_STATION_COMMAND))||
	!(strcmp(puper_str,PEEK_ONE_STATION_COMMAND))
	)
	client_read_message(s,session_keyword,0,0);
	
    if (!(strcmp(puper_str,GET_ONLY_WARRANTY_COMMAND))||
	!(strcmp(puper_str,GET_AND_ADD_WARRANTY_COMMAND))
	)
	client_read_message(s,session_keyword,1,1);
	
/*close connection*/
    close(s);
    
    return (0);
}