/*RSBAC global network client
 *
 *(c) Stanislav Ievlev <inger@linux.ru.net>
 */
 
/*auth.c - some auth functions*/

#include <time.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/socket.h>

#include <pwd.h>
#include <grp.h>
#include <gmp.h>

#include "md5.h"
#include "r-job.h"
#include "rijndael.h"

#include "../include/config.h"
#include "../include/rsa.h"

const char *tmpline="1234567890qwertyuiopasdfghjklzxcvbnmQWERTYUIOPASDFGHJKLZXCVBNM";
char *numbers="1234567890abcdefghijklmnopqrstuvwzyz";

char *get_salt(char *salt)
{
    time_t now=time(NULL);
    srand(now);
    
    salt[0]=tmpline[(random()%60)];
    salt[1]=tmpline[(random()%60)];    
    salt[2]=0;        
    
    return salt;
}

char *get_password(char *key,int length)
{
    time_t now=time(NULL);
    srand(now);
    key[length--]=0;
    while((length--)>=0)
      key[length]=tmpline[(random()%60)];
    
    return key;
}

/*get random number as a string*/
char *get_random_number(char *tmpstr,int num)
{
    int  i=0,j=0,can=0;

//    srandom(time(NULL));
    for(j=0;j<num;j++){
	tmpstr[i]=numbers[(random()%NUMBER_BASE)];
	if((tmpstr[i]=='0')&&(can==0)){
	/*skip first zeros*/
	}else{
	    can=1;/*now we can receive zeros*/
	    i++;
	}

    }
    tmpstr[i]=0; /*last symbol*/
    
    return tmpstr;
}


char *get_secret(char *key)
{
struct passwd *my_p;
struct group *my_g;

    my_p=getpwuid(getuid());
    my_g=getgrgid(getgid());
    snprintf(key,MAX_LINE,"%u%u%s%s",
    getuid(),getgid(),my_p->pw_name,my_g->gr_name);    
    
    return key;
}


char *my_hash(char *password,char *salt)
{
/*    int i;
    time_t now=time(NULL);
    struct tm *my_tm;
    
    my_tm=localtime(&now);
    
    for (i=0;i<strlen(password);i++)
	password[i]+=my_tm->tm_min;*/  /*Time dependence password: Removed!!*/

/*    password=Goodcrypt_md5(password,salt);*/
    
    return password;
}

/*==========Crypt==========*/
void my_encrypt(char *password,char *message)
{
#ifdef USE_AES_CRYPT
    int nb = 8, nk = 8,i;
    char *tmp_message;
    //new
    char local_password1[MAX_LINE],*local_password=local_password1;
    
    strncpy(local_password,password,MAX_LINE);
    
    //use password as key
    gkey (nb, nk, local_password);
    tmp_message=message;
    
    for (i=0;i<MULT;i++){
        encrypt(tmp_message);
	tmp_message+=32;
	}

#endif
}

void my_decrypt(char *password,char *message)
{
#ifdef USE_AES_CRYPT
    int nb = 8, nk = 8,i;
    char *tmp_message;
    //new
    char local_password1[MAX_LINE],*local_password=local_password1;
    strncpy(local_password,password,MAX_LINE);

    //use password as key
    gkey (nb, nk, local_password);
    tmp_message=message;
    
    for (i=0;i<MULT;i++){
        decrypt(tmp_message);
	tmp_message+=32;
	}
#endif
}
/*==========Crypt==========*/

/*==========D-H===========*/
/*helper for Diffie-Hellman algorithm*/
char *DH_exp(char *result,char *base,char *exp)
{
    mpz_t a,b,res,mod;
    char module[MAX_LINE];
    
    /*create module*/
    memset(module,'0',MAX_LINE);
    module[0]='1';
    module[MAX_LINE-1]=0;
    
    
    mpz_init(a);
    mpz_init(b);
    mpz_init(mod);
    mpz_init(res);

#ifdef TEST    
//    fprintf(stderr,">>r:%s\n>>b:%s\n>>e:%s\n>>m:%s\n",result,base,exp,module);
#endif
    mpz_set_str(a,base,NUMBER_BASE);
    mpz_set_str(b,exp,NUMBER_BASE);
    mpz_set_str(mod,module,NUMBER_BASE);
    
    mpz_powm(res,a,b,mod); //a**b mod MOD
    mpz_get_str(result,NUMBER_BASE,res);

    mpz_clear(a);
    mpz_clear(b);
    mpz_clear(res);
    mpz_clear(mod);
    
    return result;
}


/*client side of Diffie-Hellman algorithm*/
char *DH_check(int s,char *DH_keyword,int server)
{
    
    char key_local[MAX_LINE];
    char key_send[MAX_LINE];
    char key_receive[MAX_LINE];

    /*init local keyword*/
    srandom(time(NULL));
    get_random_number(key_local,MAX_LINE);
    

#ifdef TEST
//    fprintf(stderr,">>DH_local:%s\n",key_local);
#endif
   
    /*first exponent*/
    (void)DH_exp((char *)key_send,EXPONENT,(char *)key_local);    
    
    if (server==0){ /*if we are client*/
        /*send key to server*/
        if (send(s, key_send, MAX_LINE,0) ==-1){
	    perror("send");
	    exit(1);
        }
	/*recieve key from server*/
	if (recv(s, key_receive, MAX_LINE, 0)==-1){
	    perror("recv()");
	    exit(1);
	}
    }else{ /*if we are server*/
	/*recieve key from client*/
	if (recv(s, key_receive, MAX_LINE, 0)==-1){
	    perror("recv()");
	    exit(1);
	}
        /*send key to client*/
        if (send(s, key_send, MAX_LINE,0) ==-1){
	    perror("send");
	    exit(1);
        }
    }

#ifdef TEST
//    fprintf(stderr,">>DH_send:%s\n>>HD_receive:%s\n",key_send,key_receive);
#endif
    
    /*second exponent*/
    (void)DH_exp(DH_keyword,(char *)key_receive,(char *)key_local);    
    
    return DH_keyword;
}

/*==========D-H===========*/
