#include <time.h>
#include <gmp.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include <sys/types.h>
#include <sys/socket.h>

#include "r-job.h"
#include "auth.h"

char cert_filename[MAX_LINE];/*Path to certificate*/

/*Make path*/
void certificate_path(char *path)
{
    if (path==NULL){
	strncpy(cert_filename,CERTIFICATE_FILE_PATH,MAX_LINE);
    }else{
	strncpy(cert_filename,path,MAX_LINE);
    }
return;
}

/*Make name of Certificate file*/
void certificate_name(char *filename,char *station,char *type)
{
    /*name: ./certificate~127.0.0.1~public*/
    strncpy(filename,CERTIFICATE_FILE_PATH,MAX_LINE);
    strncat(filename,"~",MAX_LINE);
    strncat(filename,station,MAX_LINE);
    strncat(filename,"~",MAX_LINE);
    strncat(filename,type,MAX_LINE);

}
/*Get some random prime*/
void get_random_prime(mpz_t big,int num)
{
    char tmpstr[MAX_LINE];

    while(1)
    {
	(void)get_random_number(tmpstr,num);
#ifdef TEST
//    printf(".");
#endif
    mpz_set_str(big,tmpstr,NUMBER_BASE);
    if (mpz_probab_prime_p(big,30)==1) return;
    }
    return;
}

/*Generalized Euclid algorithm: from Knuth*/
void Euclid(mpz_t a,mpz_t m,mpz_t b,mpz_t n)
{
    mpz_t a1,b1,c,q,r,t,d,tmp1;

    mpz_init(a1);
    mpz_init(b1);
    mpz_init(c);
    mpz_init(q);
    mpz_init(r);
    mpz_init(t);
    mpz_init(d);

    mpz_init(tmp1);
/*E1*/
mpz_set_ui(a1,1);mpz_set_ui(b,1);
mpz_set_ui(a,0);mpz_set_ui(b1,0);
mpz_set(c,m);mpz_set(d,n);

while(1){
/*E2*/
mpz_tdiv_qr(q,r,c,d);


/*E3*/
if (mpz_cmp_ui(r,0)==0) break;

/*E4*/
mpz_set(c,d);mpz_set(d,r);
mpz_set(t,a1);mpz_set(a1,a);mpz_mul(tmp1,a,q);mpz_sub(a,t,tmp1);
mpz_set(t,b1);mpz_set(b1,b);mpz_mul(tmp1,b,q);mpz_sub(b,t,tmp1);

#ifdef TEST
/*fprintf(stdout,"\n");
fprintf(stdout," a1=");mpz_out_str(stdout,NUMBER_BASE,a1);
fprintf(stdout," a=");mpz_out_str(stdout,NUMBER_BASE,a);
fprintf(stdout," b1=");mpz_out_str(stdout,NUMBER_BASE,b1);
fprintf(stdout," b=");mpz_out_str(stdout,NUMBER_BASE,b);
fprintf(stdout," c=");mpz_out_str(stdout,NUMBER_BASE,c);
fprintf(stdout," d=");mpz_out_str(stdout,NUMBER_BASE,d);
fprintf(stdout," q=");mpz_out_str(stdout,NUMBER_BASE,q);
fprintf(stdout," r=");mpz_out_str(stdout,NUMBER_BASE,r);
fprintf(stdout," t=");mpz_out_str(stdout,NUMBER_BASE,t);
fprintf(stdout," qa=");mpz_out_str(stdout,NUMBER_BASE,tmp1);*/
#endif
}

mpz_clear(a1);
mpz_clear(b1);
mpz_clear(c);
mpz_clear(q);
mpz_clear(r);
mpz_clear(t);
mpz_clear(d);

mpz_clear(tmp1);
return;
}

/*Generate Public and Private keys: d,n - private e,n - public*/
void gen_keys(mpz_t d,mpz_t e,mpz_t n)
{
    mpz_t p,q,y;
    mpz_t tmp1,tmp2;

/*init numbers*/
    mpz_init(p);
    mpz_init(q);
    mpz_init(y);
    
    mpz_init(tmp1);
    mpz_init(tmp2);

    mpz_set_ui(e,2);
    
    srandom(time(NULL));
    
    /*generate primes P and Q*/
    get_random_prime(p,NUMBER_LENGTH);
    while(1){
	get_random_prime(q,NUMBER_LENGTH);
	if (mpz_cmp(p,q)!=0) break;
    }
    
    /*n=p*q*/
    mpz_mul(n,p,q);

    /*e<n and NOD(e,(p-1)*(q-1)=1)*/
    mpz_set(tmp1,p);		/* tmp1=p */
    mpz_set(tmp2,q);		/* mpt2=q */
    mpz_sub_ui(tmp1,tmp1,1);	/* tmp1=p-1 */
    mpz_sub_ui(tmp2,tmp2,1);	/* tmp2=q-1 */
    mpz_mul(tmp1,tmp1,tmp2);	/* tmp1=(p-1)*(q-1) */

    /*mpz_set(e,n);*/
    while (mpz_cmp(e,n)<0)
    {
	mpz_gcd(tmp2,e,tmp1);
        if (mpz_cmp_ui(tmp2,1)==0) break;
	mpz_add_ui(e,e,1);
    }

    /*a*m+b*n*/
    Euclid(d,e,y,tmp1); /* e*d+(p-1)*(q-1)*y=1 */

    /*(e,n) - Public key*/
    /*(d) - Private key*/
    
    /*May be d<0; we must correct it*/
    while(mpz_cmp_ui(d,0)<=0){
#ifdef TEST
	//fprintf(stdout,"\nold d:");mpz_out_str(stdout,NUMBER_BASE,d);
#endif
	mpz_add(d,d,tmp1); /*d=d+(n)*/
    };

#ifdef TEST
    fprintf(stdout,"\n");
    fprintf(stdout,"p:");mpz_out_str(stdout,NUMBER_BASE,p);
    fprintf(stdout,"\nq:");mpz_out_str(stdout,NUMBER_BASE,q);
    fprintf(stdout,"\nn:");mpz_out_str(stdout,NUMBER_BASE,n);
    fprintf(stdout,"\ne:");mpz_out_str(stdout,NUMBER_BASE,e);
    fprintf(stdout,"\n");
    fprintf(stdout,"\ntmp1:");mpz_out_str(stdout,NUMBER_BASE,tmp1);
    fprintf(stdout,"\ntmp(NOD):");mpz_out_str(stdout,NUMBER_BASE,tmp2);
    fprintf(stdout,"\nd:");mpz_out_str(stdout,NUMBER_BASE,d);
    fprintf(stdout,"\ny:");mpz_out_str(stdout,NUMBER_BASE,y);
    fprintf(stdout,"\n");
#endif

/*clear numbers*/
    mpz_clear(p);
    mpz_clear(q);
    mpz_clear(y);
    
    mpz_clear(tmp1);
    mpz_clear(tmp2);
    return;
}

/*Generate Certificate's files*/
void gen_certificates(char *station)
{
    mpz_t d,e,n;
    FILE *f=NULL;
    char filename[MAX_LINE];
    
    mpz_init(d);
    mpz_init(e);
    mpz_init(n);
    
    gen_keys(d,e,n);
    
    /*name: ./certificate~127.0.0.1~private*/
    certificate_name(filename,station,"private");
    
    f=fopen(filename,"w");
    mpz_out_str(f,NUMBER_BASE,d);
    fclose(f);
    
    /*name: ./certificate~127.0.0.1~public*/
    certificate_name(filename,station,"public");    
    
    f=fopen(filename,"w");
    mpz_out_str(f,NUMBER_BASE,e);
    fclose(f);

    /*name: ./certificate~127.0.0.1~module*/
    certificate_name(filename,station,"module");        
    
    f=fopen(filename,"w");
    mpz_out_str(f,NUMBER_BASE,n);
    fclose(f);
    
    mpz_clear(d);
    mpz_clear(e);
    mpz_clear(n);

    return;
}

/*RSA encrypt using Public key*/
void RSA_Public_encrypt(char *station,char *message)
{
    mpz_t e,n,c;
    FILE *f=NULL;
    char tmpstr[MAX_LINE];
    char filename[MAX_LINE];

    mpz_init(e);
    mpz_init(n);
    mpz_init(c);
    
    /*open key files*/
    /*name: ./certificate~127.0.0.1~public*/
    certificate_name(filename,station,"public");        
    
    f=fopen(filename,"r");
    if (f==NULL){
    perror("fopen");
    fprintf(stderr,"Create certificate file using getkey utility\n");
    exit(1);
    }
    memset(tmpstr,0,MAX_LINE);
    fread((char*)tmpstr,MAX_LINE,1,f);
    mpz_set_str(e,tmpstr,NUMBER_BASE);
    fclose(f);

    /*name: ./certificate~127.0.0.1~module*/
    certificate_name(filename,station,"module");            
    
    f=NULL;
    f=fopen(filename,"r");
    if (f==NULL){
    perror("fopen");
    fprintf(stderr,"Create certificate file using getkey utility\n");
    exit(1);
    }

    memset(tmpstr,0,MAX_LINE);
    fread((char*)tmpstr,MAX_LINE,1,f);
    mpz_set_str(n,tmpstr,NUMBER_BASE);
    fclose(f);

    /*setup message*/
    mpz_set_str(c,message,NUMBER_BASE);
    
#ifdef TEST
/*    printf("\n");
    fprintf(stdout," e=");mpz_out_str(stdout,NUMBER_BASE,e);
    fprintf(stdout," n=");mpz_out_str(stdout,NUMBER_BASE,n);
    fprintf(stdout," c=");mpz_out_str(stdout,NUMBER_BASE,c);
    printf("\n");*/
#endif

    /*encrypt*/
    mpz_powm(c,c,e,n); /* c=m^e mod n */
#ifdef TEST
//    fprintf(stdout," c=");mpz_out_str(stdout,NUMBER_BASE,c);
#endif
    mpz_get_str(message,NUMBER_BASE,c);
    
    mpz_clear(e);
    mpz_clear(n);
    mpz_clear(c);

}

/*RSA decrypt using Private key*/
void RSA_Private_decrypt(char *station,char *message)
{
    mpz_t d,n,c;
    FILE *f=NULL;
    char tmpstr[MAX_LINE];
    char filename[MAX_LINE];

    mpz_init(d);
    mpz_init(n);
    mpz_init(c);
    
    /*open key files*/
    /*name: ./certificate~127.0.0.1~private*/
    certificate_name(filename,station,"private");        
    
    f=fopen(filename,"r");
    if (f==NULL){
    perror("fopen");
    fprintf(stderr,"Create certificate file using getkey utility\n");
    exit(1);
    }

    memset(tmpstr,0,MAX_LINE);
    fread((char*)tmpstr,MAX_LINE,1,f);
    mpz_set_str(d,tmpstr,NUMBER_BASE);
    fclose(f);

    /*name: ./certificate~127.0.0.1~module*/
    certificate_name(filename,station,"module");            
    
    f=NULL;
    f=fopen(filename,"r");
    if (f==NULL){
    perror("fopen");
    fprintf(stderr,"Create certificate file using getkey utility\n");
    exit(1);
    }

    memset(tmpstr,0,MAX_LINE);
    fread((char*)tmpstr,MAX_LINE,1,f);
    mpz_set_str(n,tmpstr,NUMBER_BASE);
    fclose(f);

    /*setup message*/
    mpz_set_str(c,message,NUMBER_BASE);
    
#ifdef TEST
/*    printf("\n");
    fprintf(stdout," d=");mpz_out_str(stdout,NUMBER_BASE,d);
    fprintf(stdout," n=");mpz_out_str(stdout,NUMBER_BASE,n);
    fprintf(stdout," c=");mpz_out_str(stdout,NUMBER_BASE,c);
    printf("\n");*/
#endif

    /*encrypt*/
    mpz_powm(c,c,d,n); /* c=m^d mod n */
#ifdef TEST
//    fprintf(stdout," c=");mpz_out_str(stdout,NUMBER_BASE,c);
#endif
    mpz_get_str(message,NUMBER_BASE,c);
    
    mpz_clear(d);
    mpz_clear(n);
    mpz_clear(c);
}

/*----------------Client-Server-Work--------------------*/
/*Send client's public key to server*/
void client_accept_station(int s,char *station)
{
    FILE *f=NULL;
    char tmpstr[MAX_LINE];
    char filename[MAX_LINE];
    
    /*name: ./certificate~127.0.0.1~public*/
    certificate_name(filename,station,"public");        
    f=fopen(filename,"r");
    if (f==NULL){
    perror("fopen");
    fprintf(stderr,"Create certificate file using getkey utility\n");
    exit(1);
    }

    memset(tmpstr,0,MAX_LINE);
    fread((char*)tmpstr,MAX_LINE,1,f);
    fclose(f);
    
    if (send(s, tmpstr, MAX_LINE,0) ==-1){
        perror("send");
	exit(1);
    }
#ifdef TEST
//    printf(">>RSA_send_p:%s\n",tmpstr);
#endif

    /*name: ./certificate~127.0.0.1~module*/
    certificate_name(filename,station,"module");        
    f=NULL;
    f=fopen(filename,"r");
    if (f==NULL){
    perror("fopen");
    fprintf(stderr,"Create certificate file using getkey utility\n");
    exit(1);
    }

    memset(tmpstr,0,MAX_LINE);
    fread((char*)tmpstr,MAX_LINE,1,f);
    fclose(f);
    
    if (send(s, tmpstr, MAX_LINE,0) ==-1){
        perror("send");
	exit(1);
    }
#ifdef TEST
//    printf(">>RSA_send_m:%s\n",tmpstr);
#endif
}

/*Receive client's public key to server*/
void server_accept_station(int s,char *station)
{
    FILE *f=NULL;
    char tmpstr[MAX_LINE],tmpstr_m[MAX_LINE];
    char filename[MAX_LINE];
    
    if (recv(s, tmpstr, MAX_LINE,0) ==-1){ /*receive key*/
        perror("recieve");
	exit(1);
    }
    if (recv(s, tmpstr_m, MAX_LINE,0) ==-1){/*receive module*/
        perror("recieve");
	exit(1);
    }

    certificate_name(filename,station,"public");        
    f=fopen(filename,"r");
    
    if (f==NULL) /*don't meet this station yet*/
    {
#ifdef TEST
//    printf(">>RSA:first meeting\n");
#endif
    f=fopen(filename,"w");
    fwrite(tmpstr,MAX_LINE,1,f);
    fclose(f);
    
    certificate_name(filename,station,"module");        
    f=fopen(filename,"w");
    fwrite(tmpstr_m,MAX_LINE,1,f);
    fclose(f);
    }else{ /*we meet this station before*/
#ifdef TEST
//    printf(">>RSA:we meet before\n");
#endif
    fclose(f);
    }
    
#ifdef TEST
//printf(">>RSA:receive_p:%s\n",tmpstr);
//printf(">>RSA:receive_m:%s\n",tmpstr_m);
#endif
}

/*Test station */
int server_test(int s,char *station)
{
    char message[MAX_LINE];
    char teststr[MAX_LINE];
    
    /*make test message*/
    srandom(time(NULL));
    get_random_number(teststr,NUMBER_LENGTH);
    
    /*Encrypt using client's public key*/
    strncpy(message,teststr,MAX_LINE);
    (void)RSA_Public_encrypt(station,message);
    
    /*Send question*/
    if (send(s, message, MAX_LINE,0) ==-1){
        perror("send");
	exit(1);
    }
    
#ifdef TEST
    fprintf(stderr,">>RSA:test:%s\n",teststr);
//    fprintf(stderr,">>RSA:test_send:%s\n",message);
#endif
    /*Recieve answer*/
    if (recv(s, message, MAX_LINE,0) ==-1){
        perror("receive");
	exit(1);
    }
    
    /*Check it*/
    if (!(strncmp(teststr,message,MAX_LINE))){
#ifdef TEST
//        fprintf(stderr,">>RSA:ok\n");
#endif
    return 1;
    }else{
#ifdef TEST
//        fprintf(stderr,">>RSA:failed\n");
#endif
    return 0;
    }
}

/*Test server */
void client_test(int s,char *station)
{
    char message[MAX_LINE];
    
    /*Receive message*/
    if (recv(s, message, MAX_LINE,0) ==-1){
        perror("recieve");
	exit(1);
    }
    
#ifdef TEST
//    fprintf(stderr,">>RSA:test_receive:%s\n",message);
#endif

    /*Decrypt using private key*/
    (void)RSA_Private_decrypt(station,message);
    /*Send answer*/
    if (send(s, message, MAX_LINE,0) ==-1){
        perror("send");
	exit(1);
    }
    
#ifdef TEST
//    fprintf(stderr,">>RSA:test_result:%s\n",message);
#endif
}

void client_check_cert()
{
FILE *f=NULL;
char filename[MAX_LINE];
    certificate_name(filename,"client","public");        
    f=fopen(filename,"r");
    if(f==NULL){
	perror("fopen");
	fprintf(stderr,"Create certificate using getkeys utility first\n");
	exit(1);
    }
}