/*RSBAC global network
 *
 *(c) Stanislav Ievlev <inger@linux.ru.net>
 */

/*r-jobd-clist.c - jobs server.Some work with station list*/

#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <fcntl.h>
#include <netdb.h>
#include <syslog.h>
#include <signal.h>
#include <string.h>

#include "r-job.h"
#include "r-job-d.h"
#include "auth.h"

/*helpers*/

/* current format  level:station */

extern char *clientlist_filename;

/*get name of station*/
char *get_station_name(char *name,char *station)
{
    char *find=NULL;

    find=strchr(station,':');
    if (find!=NULL) {
	find++;
	strncpy(name,find,MAX_LINE);
    }
    if (name[strlen(name)-1]==10)
	    name[strlen(name)-1]=0; /*remove CR-LF if needed*/
    return name;
}

/*get level of station*/
int get_station_level(char *station)
{
    int res=0;
    FILE *cl_file=NULL;
    char tmpstr[MAX_LINE];
    char tmpstr1[MAX_LINE];
    //char *find;
    
    cl_file=fopen(/*CLIENT_LIST_PATH*/clientlist_filename,"r");
    
    if (cl_file==NULL)
    {
	syslog(LOG_INFO, "rsbac-net: Cannot open client-list file.");
	exit(1);
    }
    
    while (fgets(tmpstr,MAX_LINE,cl_file)!=NULL)
	if (!(strcmp(get_station_name(tmpstr1,tmpstr),station))){
	    res=atoi(tmpstr);/*get level*/
	    break;
	}
    
    fclose(cl_file);
    
    return res;
}


/*append new station to clients list*/
void  append_station(char *station)
{
    FILE *cl_file=NULL;
    
    if (in_station_list(station)==1){
#ifdef TEST
	fprintf(stderr,">>station already exists\n");
#endif
    return;
    }
    
    cl_file=fopen(clientlist_filename,"a");
    if (cl_file==NULL)
    {
	syslog(LOG_INFO, "rsbac-net: Cannot open client-list file.");
	exit(1);
    };
    fprintf(cl_file,"0:%s\n",station);
    fclose(cl_file);
        
    return;
}

/*remove station from clients list*/
void  remove_station(char *station)
{
    FILE *cl_file=NULL,*tmp_file=NULL;
    char tmpstr[MAX_LINE];
    char tmpstr1[MAX_LINE];
    
    char tmp_filename[MAX_LINE]; /*tmp file*/
    
    strncpy(tmp_filename,clientlist_filename,MAX_LINE);
    strncat(tmp_filename,"~tmp",MAX_LINE);
    
    cl_file=fopen(clientlist_filename,"r"); /*client list*/
    tmp_file=fopen(tmp_filename,"w"); /*tmp file*/
    
    if ((cl_file==NULL)||(tmp_file==NULL))
    {
	syslog(LOG_INFO, "rsbac-net: Cannot open client-list or temp file.");
	exit(1);
    };
    
    while (fgets(tmpstr,MAX_LINE,cl_file)!=NULL)
    {

        if (!(strncmp(get_station_name(tmpstr1,tmpstr),station,MAX_LINE))){ /*station was found*/
#ifdef TEST
	    printf(">>station found\n");
#endif    
	/*skip put this line to tmp file*/
	}else{ /*put all other stations back to list*/
	fprintf(tmp_file,"%s",tmpstr);
	}
    };
    
    fclose(cl_file);
    fclose(tmp_file);
    
    rename(tmp_filename,clientlist_filename); /*make tmp file client's list*/
    
    return;
}

/*check for station in station list*/
int in_station_list(char *station)
{
    int res=0;
    FILE *cl_file=NULL;
    char tmpstr[MAX_LINE];
    char tmpstr1[MAX_LINE];
    
    cl_file=fopen(clientlist_filename,"r");
    
    if (cl_file==NULL)
    {
	syslog(LOG_INFO, "rsbac-net: Cannot open client-list file.");
	exit(1);
    }
    
    while (fgets(tmpstr,MAX_LINE,cl_file)!=NULL)
    {
	//tmpstr[strlen(tmpstr)-1]=0;
	
	if (!(strcmp(get_station_name(tmpstr1,tmpstr),station)))
	     res=1;
    }
    
    fclose(cl_file);
    
    return res;
}

/*send station list to client*/
void send_stations(int ns,char *session_keyword,char *crypto_keyword)
{
FILE *c_file=NULL;
char data_str[MAX_LINE];
char data_str1[MAX_LINE];

    c_file=fopen(/*CLIENT_LIST_PATH*/clientlist_filename,"r");
    
    if (c_file!=NULL){
    
	while (fgets(data_str,MAX_LINE,c_file)!=NULL){
	
	    get_station_name(data_str1,data_str);
#ifdef TEST
	    printf(">>send:%s\n",data_str1);
#endif

	    my_encrypt(session_keyword,data_str1); /*re-crypt using per session keyword*/
	    if (send(ns, data_str1, MAX_LINE, 0)==-1){/*send data to client*/
		syslog(LOG_INFO, "rsbac-net: Cannot send message.");
    	    }
	}/*while*/
	
	fclose(c_file);    
    }/*if*/
    
}
